/*************************************************
* DSA Header File                                *
* (C) 1999-2002 The Botan Project                *
*************************************************/

#ifndef BOTAN_DSA_H__
#define BOTAN_DSA_H__

#include <botan/bigint.h>
#include <botan/pk_keys.h>
#include <botan/mod_exp.h>
#include <botan/dl_param.h>

namespace Botan {

/*************************************************
* DSA Public Key                                 *
*************************************************/
class DSA_PublicKey : public PK_Verifying_wo_MR_Key, public X509_PublicKey
   {
   public:
      bool verify(const byte[], u32bit, const byte[], u32bit) const;

      u32bit max_input_bits() const;
      bool check_params() const;

      const DL_Group& get_domain() const { return group; }
      const BigInt& get_y() const { return y; }

      DSA_PublicKey(const DL_Group&, const BigInt&);
      DSA_PublicKey(DataSource&, X509_Encoding = PEM);
   protected:
      SecureVector<byte> encode_key_x509() const;
      SecureVector<byte> encode_algo_params() const;
      void decode_key_x509(DataSource&);
      void decode_algo_params(DataSource&);
      OID get_oid() const;
      DL_Group group;
      BigInt y;
      FixedBase_Exp powermod_g_p, powermod_y_p;
   };

/*************************************************
* DSA Private Key                                *
*************************************************/
class DSA_PrivateKey : public DSA_PublicKey, public PK_Signing_Key
   {
   public:
      SecureVector<byte> sign(const byte[], u32bit) const;

      bool check_params() const;
      const BigInt& get_x() const { return x; }

      DSA_PrivateKey(const DL_Group&);
      DSA_PrivateKey(const DL_Group&, const BigInt&, const BigInt&);
   private:
      BigInt x;
   };

}

#endif
