/*************************************************
* X.509 Header File                              *
* (C) 1999-2002 The Botan Project                *
*************************************************/

#ifndef BOTAN_X509_H__
#define BOTAN_X509_H__

#include <botan/pipe.h>
#include <botan/data_src.h>
#include <botan/asn1_oid.h>

namespace Botan {

/*************************************************
* X.509 Public Key                               *
*************************************************/
class X509_PublicKey
   {
   public:
      void X509_encode(Pipe&, X509_Encoding = PEM) const;
      void X509_decode(DataSource&, X509_Encoding = PEM);

      std::string PEM_encode() const;
      SecureVector<byte> DER_encode() const;
   private:
      void PEM_decode(DataSource&);
      void BER_decode(DataSource&);

      virtual SecureVector<byte> encode_key_x509() const = 0;
      virtual SecureVector<byte> encode_algo_params() const;

      virtual void decode_key_x509(DataSource&) = 0;
      virtual void decode_algo_params(DataSource&);

      virtual OID get_oid() const = 0;
      virtual bool valid_oid(const OID&) const;
   };

}

#endif
