/*************************************************
* ASN.1 OID Source File                          *
* (C) 1999-2002 The Botan Project                *
*************************************************/

#include <botan/asn1_oid.h>
#include <botan/util.h>

namespace Botan {

/*************************************************
* Return this OID as a string                    *
*************************************************/
std::string OID::as_string() const
   {
   std::string oid_str;
   for(u32bit j = 0; j != id.size(); j++)
      {
      oid_str += to_string(id[j]);
      if(j != id.size() - 1)
         oid_str += '.';
      }
   return oid_str;
   }

/*************************************************
* OID equality comparison                        *
*************************************************/
bool OID::operator==(const OID& oid) const
   {
   if(id.size() != oid.id.size())
      return false;
   for(u32bit j = 0; j != id.size(); j++)
      if(id[j] != oid.id[j])
         return false;
   return true;
   }

/*************************************************
* Append another component to the OID            *
*************************************************/
OID& OID::operator+=(u32bit component)
   {
   id.push_back(component);
   return (*this);
   }

/*************************************************
* ASN.1 OID Constructor                          *
*************************************************/
OID::OID(const std::string& oid_str)
   {
   if(oid_str != "")
      {
      id = parse_asn1_oid(oid_str);
      if(id.size() < 2 || id[0] > 2)
         throw Invalid_OID(oid_str);
      if((id[0] == 0 || id[0] == 1) && id[1] > 39)
         throw Invalid_OID(oid_str);
      }
   }

}
